Program IMV;
{ Inertia Mesh Viewer for TMT Pascal }

{$R-}
{$S-}
{$Q-}

Uses
  Types,
  Inertia,
  VBE,
  Menu,
  Crt,
  TrkBall,
  Mouse;

Const
  ObjectList : Array[0..4] of String = (
    'TORUS.GVO',
    'HEAD.GVO',
    'CHRMFACE.GVO',
    'SKULL.GVO',
    'DUCK.GVO');

  DataSet : String = 'Copprenv';

  TransFlag   : Longint = 0;
  SurfaceFlag : Longint = SMOOTH;
  ShadeFlag   : Longint = LAMBERT;

  ShadingTable : Array[0..4] of Longint = (
    UNSHADED,
    AMBIENT,
    LAMBERT,
    GOURAUD,
    PHONG);

  SurfaceTable : Array[0..2] of Longint = (
    SMOOTH,
    TEXTURE,
    REFLECTION);

  XRes : Longint = 320;
  YRes : Longint = 200;

Var
  View   : ViewObject;
  Obj    : VectorObject;
  ZLoc   : Real;
  zscale : Real;


Procedure ObjectCallback     (Var Data : Longint); Forward;
Procedure SurfaceCallback    (Var Data : Longint); Forward;
Procedure ShadeCallback      (Var Data : Longint); Forward;
Procedure TransparentCallback(Var Data : Longint); Forward;
Procedure QuitCallback       (Var Data : Longint); Forward;
Procedure InitMenu;                                Forward;
Procedure Init3D;                                  Forward;
Procedure InitMisc;                                Forward;
Procedure TextQuit           (S : String);         Forward;
Procedure RunApplication;                          Forward;

Procedure PutPixel      (x, y, c : Longint);       Forward;
Procedure PutMouseCursor(x, y : Longint);          Forward;



Procedure Main;

Begin
  Writeln('Initializing');

  InitMenu;
  Init3D;
  InitMisc;

  RunApplication;
End;

{===========================================================================
  Initializes the menu system
 ===========================================================================}
Procedure InitMenu;

Var
  i : Longint;

Begin
  SubMenu('Objects');
    For i := 0 to 4 do
      MenuItem(ObjectList[i], ObjectCallback, i);
  EndSubMenu;

  SubMenu('Surface');
    MenuItem('Smooth',     SurfaceCallback, 0);
    MenuItem('Texture',    SurfaceCallback, 1);
    MenuItem('Reflection', SurfaceCallback, 2);
  EndSubMenu;

  SubMenu('Shading');
    MenuItem('Unshaded',   ShadeCallback, 0);
    MenuItem('Ambient',    ShadeCallback, 1);
    MenuItem('Lambert',    ShadeCallback, 2);
    MenuItem('Gouraud',    ShadeCallback, 3);
    MenuItem('Phong',      ShadeCallback, 4);
  EndSubMenu;

  ToggleMenuItem('Transparent', TransparentCallback, 0);

  MenuItem('Quit',  QuitCallback, 0);

  EndMenu;
End;

{===========================================================================
  Initializes the 3D system
 ===========================================================================}
Procedure Init3D;

Begin
  InitInertia;
  If (InitMouse <> -1) Then TextQuit('A mouse is required to run this program.');
  If LoadIntensityTable(DataSet + '.It', 0) <> I_OK   Then TextQuit('Error Loading Intensity Table');
  If LoadTransparencyTable(DataSet + '.tt', 0) <> I_OK Then TextQuit('Error Loading Transparency Table');
  If LoadPCXTexture(1, DataSet + '.PCX', 0) <> I_OK   Then TextQuit('Error Loading Texture');
  If MakePhongMap(0) <> 0 Then TextQuit('Error making phong buffer');

  InitView(View);

  SetViewPort(View, 0, 0, XRes - 1, YRes - 1, True);

  GetTexturePalette(1);
  PushLightSource(0, 0, -1);
End;

{===========================================================================
  Initializes a few other things
 ===========================================================================}
Procedure InitMisc;

Var
  Mode : Word;
  i : Word;

Begin
  if (Not(InitVESA))
    Then Begin
      Writeln('Vesa driver not found!');
      halt(0);
    End;

  Mode := SearchMode(XRes, YRes, 8);

  if (Mode = $FFFF)
    Then Begin
      Writeln('Error! VESA ', XRes, 'x', YRes, ' mode could not be initialized.');
      Halt(0);
    End;

  SetMode(Mode);

  InitMouse;

  For i := 0 to 255 do
    Begin
      SetRGB(i, GlobalPalette[i].r, GlobalPalette[i].g, GlobalPalette[i].b);
    End;

  Getmem(VirtualPage, XRes * YRes);
  Fillchar(VirtualPage^, XRes * YRes, 0);

  SetVirtualPage(VirtualPage);
  SetScreenSize(XRes, XRes);
  SetMenuColors(253, 254, 255);

  SetRGB(253, 63, 63, 63);
  SetRGB(254, 45, 45, 45);
  SetRGB(255, 32, 32, 32);

  SetRange(0, 0, XRes - 1, YRes - 1);
  SetMouseLocation(XRes Div 2, YRes Div 2);
End;

{===========================================================================
  Main application even loop
 ===========================================================================}
Procedure RunApplication;

Var
  x, y, b : Integer;
  ox, oy : Longint;
  Event, ButtonFlag : Longint;
  rotx, roty : Integer;

Begin
  ox := -1;
  oy := -1;

  ButtonFlag := 0;
  Event := 0;

  ReadMouse(x, y, b);

  Repeat
    ReadMickey(rotx, roty);

    Inc(x, rotx);
    Inc(y, roty);

    if (x > (XRes - 1)) Then x := XRes - 1;
    if (x < 0)   Then x := 0;
    if (y > (YRes - 1)) Then y := YRes - 1;
    if (y < 0)   Then y := 0;

    if LeftButtonPressed
      Then Begin
        Event := 1;
        ButtonFlag := ButtonFlag or MOUSE_BUTTON_1_PRESSED;
      End;

    if RightButtonPressed
      Then Begin
        Event := 1;
        ButtonFlag := ButtonFlag or MOUSE_BUTTON_2_PRESSED;
      End;

    if LeftButtonReleased
      Then Begin
        Event := 1;
        ButtonFlag := ButtonFlag or MOUSE_BUTTON_1_RELEASED;
      End;

    if RightButtonReleased
      Then Begin
        Event := 1;
        ButtonFlag := ButtonFlag or MOUSE_BUTTON_2_RELEASED;
      End;

    if ((ButtonFlag and MOUSE_BUTTON_1_PRESSED) > 0)
      Then Begin
        ZLoc := ZLoc + roty * zscale;
        SetAbsoluteLocation(Obj, 0, 0, ZLoc);
        SetObjectMatrix(Obj, 0, 0, rotx / 256.0);
      End;

    if ((rotx <> ox) or (roty <> oy))
      Then Begin
        Event := 1;
      End;

    SetObjectMatrix(Obj, rotx / 256.0, roty / 256.0, 0);

    ReadMouse(x, y, b);
    AddToRenderList(View, Obj);
    Render(VirtualPage, XRes, YRes);
    MenuAction(ButtonFlag, x, y);
    PutMouseCursor(x-4, y-4);

    Move(VirtualPage^, Mem[PScreen], XRes * YRes);
    Fillchar(VirtualPage^, XRes * YRes, 0);

    Event := 0;
    ButtonFlag := 0;
    ox    := rotx;
    oy    := roty;
  Until KeyPressed;

  DeInitVesa;
End;

Procedure TextQuit           (S : String);

Begin
  Writeln(S);
  ShutdownInertia;
  Halt(0);
End;

Procedure ObjectCallback     (Var Data : Longint);

Begin
  FreeVectorObject(Obj);

  if (LoadGVO(Obj, ObjectList[Data], 0) <> I_OK)
    Then Begin
      FreeVectorObject(Obj);
      Exit;
    End;

  SetRenderMask(Obj, TransFlag or ShadeFlag or SurfaceFlag);

  if (TransFlag <> 0)
    Then SetCull(Obj, 0);

  SetBaseColor(Obj, 36);
  SetObjectTexture(Obj, 1);
  SetObjectPhongMap(Obj, 0);

  SetAbsoluteLocation(Obj, 0, 0, 3 * Obj.Radius);

  ZLoc   := 3 * Obj.Radius;
  ZScale := Obj.Radius / 20;
End;

Procedure SurfaceCallback    (Var Data : Longint);

Begin
  SurfaceFlag := SurfaceTable[Data];

  SetRenderMask(Obj, TransFlag or ShadeFlag or SurfaceFlag);
End;

Procedure ShadeCallback      (Var Data : Longint);

Begin
  ShadeFlag := ShadingTable[Data];

  SetRenderMask(Obj, TransFlag or ShadeFlag or SurfaceFlag);
End;

Procedure TransparentCallback(Var Data : Longint);

Begin
  Data := 1 - Data;
  TransFlag := Data * TRANSPARENT;

  if (TransFlag = TRANSPARENT)
    Then SetCull(Obj, 0)
    Else SetCull(Obj, 1);

  SetRenderMask(Obj, TransFlag or ShadeFlag or SurfaceFlag);
End;

Procedure QuitCallback       (Var Data : Longint);

Begin
  DeInitVesa;
  ShutdownInertia;
  Writeln('Inertia Mesh Viewer');
  Writeln;
  Writeln('Program Code: Alex Chalfin');
  Writeln('Inertia Code: Alex Chalfin, Jeroen Bouwens');
  Writeln;
  Writeln('Graphics mode options:');
  Writeln('IMV [mode]');
  Writeln('  1 - 320x200 (default)');
  Writeln('  2 - 640x480');
  Writeln('  3 - 800x600');
  Writeln;
  Halt(0);
End;

Const
  MouseCursor : Array[0..7*7 - 1] of Byte = (
  0,0,0,1,0,0,0,
  0,0,0,1,0,0,0,
  0,0,1,1,1,0,0,
  1,1,1,0,1,1,1,
  0,0,1,1,1,0,0,
  0,0,0,1,0,0,0,
  0,0,0,1,0,0,0);

Procedure PutPixel(x, y, c : Longint);

Var
  Addr : DWord;

Begin
  if (x < 0) or (x >= XRes) Then Exit;
  if (y < 0) or (y >= YRes) Then Exit;
  if (c = 0) Then Exit;

  Addr := DWord(VirtualPage);
  Addr := Addr + y*XRes+x;

  Mem[Addr] := c;
End;

Procedure PutMouseCursor(x, y : Longint);

Var
  i, j : Longint;

Begin
  For i := 0 to 6 do
    For j := 0 to 6 do
      PutPixel(x + j, y + i, MouseCursor[i*7+j]);
End;


Begin


  case ParamStr(1)[1] of
    '1' : Begin XRes := 320; YRes := 200; End;
    '2' : Begin XRes := 640; YRes := 480; End;
    '3' : Begin XRes := 800; YRes := 600; End;
    '4' : Begin XRes := 1024; YRes := 768; End;
  End;

  Main;
End.
