Unit TrkBall;

Interface

Uses
  Types,
  Inertia;

Procedure SetObjectMatrix(Var Obj : VectorObject; x, y, z : Real);

Implementation

Type
  fVector3d = Record
    x, y, z : Real;
  End;

Var
  Xaxis, Yaxis, Zaxis : FVector3d;

Procedure AxisRotate(Var v : fVector3d; A : Real; x, y, z : Real);

Var
  cosa, sina : Real;
  dot, dx, dy, dz : Real;
  rx, ry, rz : Real;
  cx, cy, cz : Real;

Begin
  cosa := cos(a);
  sina := sin(a);
  dot := (x * v.x + y * v.y + z * v.z);
  dx := dot * x * (1 - cosa);
  dy := dot * y * (1 - cosa);
  dz := dot * z * (1 - cosa);
  rx := v.x * cosa;
  ry := v.y * cosa;
  rz := v.z * cosa;
  cx := (y * v.z - z * v.y)*sina;
  cy := (z * v.x - x * v.z)*sina;
  cz := (x * v.y - y * v.x)*sina;

  v.x := rx + dx + cx;
  v.y := ry + dy + cy;
  v.z := rz + dz + cz;
End;

Procedure FixAxis;

Var
  dot1, dot2 : Real;
  len : Real;

Begin
  dot1 := xaxis.x * yaxis.x + xaxis.y * yaxis.y + xaxis.z * yaxis.z;
  dot2 := xaxis.x * zaxis.x + xaxis.y * zaxis.y + xaxis.z * zaxis.z;
  xaxis.x := xaxis.x - dot1 * yaxis.x;
  xaxis.y := xaxis.y - dot1 * yaxis.y;
  xaxis.z := xaxis.z - dot1 * yaxis.z;
  xaxis.x := xaxis.x - dot2 * zaxis.x;
  xaxis.y := xaxis.y - dot2 * zaxis.y;
  xaxis.z := xaxis.z - dot2 * zaxis.z;
  len := 1/sqrt(sqr(xaxis.x) + sqr(xaxis.y) + sqr(xaxis.z));
  xaxis.x := xaxis.x*len;
  xaxis.y := xaxis.y*len;
  xaxis.z := xaxis.z*len;

  dot1 := xaxis.x * yaxis.x + xaxis.y * yaxis.y + xaxis.z * yaxis.z;
  dot2 := yaxis.x * zaxis.x + yaxis.y * zaxis.y + yaxis.z * zaxis.z;
  yaxis.x := yaxis.x - dot1 * xaxis.x;
  yaxis.y := yaxis.y - dot1 * xaxis.y;
  yaxis.z := yaxis.z - dot1 * xaxis.z;
  yaxis.x := yaxis.x - dot2 * zaxis.x;
  yaxis.y := yaxis.y - dot2 * zaxis.y;
  yaxis.z := yaxis.z - dot2 * zaxis.z;
  len := 1/sqrt(sqr(yaxis.x) + sqr(yaxis.y) + sqr(yaxis.z));
  yaxis.x := yaxis.x*len;
  yaxis.y := yaxis.y*len;
  yaxis.z := yaxis.z*len;

  zaxis.x := xaxis.y*yaxis.z - xaxis.z*yaxis.y;
  zaxis.y := xaxis.z*yaxis.x - xaxis.x*yaxis.z;
  zaxis.z := xaxis.x*yaxis.y - xaxis.y*yaxis.x;
End;

Procedure SetObjectMatrix(Var Obj : VectorObject; x, y, z : Real);

Var
  phi, theta : Real;

Begin
  phi := x;
  theta := y;
  AxisRotate(XAxis, theta, xaxis.x, xaxis.y, xaxis.z);
  AxisRotate(YAxis, theta, xaxis.x, xaxis.y, xaxis.z);
  AxisRotate(ZAxis, theta, xaxis.x, xaxis.y, xaxis.z);

  AxisRotate(XAxis, phi, yaxis.x, yaxis.y, yaxis.z);
  AxisRotate(YAxis, phi, yaxis.x, yaxis.y, yaxis.z);
  AxisRotate(ZAxis, phi, yaxis.x, yaxis.y, yaxis.z);

  phi := z;
  AxisRotate(XAxis, phi, zaxis.x, zaxis.y, zaxis.z);
  AxisRotate(YAxis, phi, zaxis.x, zaxis.y, zaxis.z);
  AxisRotate(ZAxis, phi, zaxis.x, zaxis.y, zaxis.z);

  FixAxis;

  Obj.Orientation[0] := Xaxis.x;
  Obj.Orientation[3] := Xaxis.y;
  Obj.Orientation[6] := Xaxis.z;

  Obj.Orientation[1] := Yaxis.x;
  Obj.Orientation[4] := Yaxis.y;
  Obj.Orientation[7] := Yaxis.z;

  Obj.Orientation[2] := Zaxis.x;
  Obj.Orientation[5] := Zaxis.y;
  Obj.Orientation[8] := Zaxis.z;
End;

Begin
  Xaxis.x := 1; Xaxis.y := 0; Xaxis.z := 0;
  Yaxis.x := 0; Yaxis.y := 1; Yaxis.z := 0;
  Zaxis.x := 0; Zaxis.y := 0; Zaxis.z := 1;
End.